@@ -359,7 +359,7 @@ int cond_normalize_expr(policydb_t * p, cond_node_t * cn)
 	ne = NULL;
 	e = cn->expr;
 
-	/* becuase it's RPN look at last element */
+	/* because it's RPN look at last element */
 	while (e->next != NULL) {
 		ne = e;
 		e = e->next;
@@ -551,6 +551,9 @@ int cond_index_bool(hashtab_key_t key, hashtab_datum_t datum, void *datap)
 	if (!booldatum->s.value || booldatum->s.value > p->p_bools.nprim)
 		return -EINVAL;
 
+	if (p->p_bool_val_to_name[booldatum->s.value - 1] != NULL)
+		return -EINVAL;
+
 	p->p_bool_val_to_name[booldatum->s.value - 1] = key;
 	p->bool_val_to_struct[booldatum->s.value - 1] = booldatum;
 
